"""
OPTIONS

some parameters for the creation of styles
and formatting.
"""

INVISIBLE_SEPARATOR = ' '
MARGIN = 1
ESCAPE_CODES = ['\n', '\r']
MIN_COLUMN_WIDTH = 2
NONE_VALUE_REPLACEMENT = ''
DEFAULT_FILL_CHAR = ' '
DEFAULT_STYLE = 'grid_eheader'
